//This code was wirtten using https://github.com/JoeyDeVries/LearnOpenGL/blob/master/src/6.pbr/1.2.lighting_textured/1.2.pbr.fs

#version 430 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 brightColor;

uniform vec3 camera_world;

uniform bool fogActive;

uniform sampler2D albedoTexture;
uniform sampler2D metallicTexture;
uniform sampler2D roughTexture;
uniform sampler2D aoTexture;
uniform sampler2D normalTexture;
uniform int textureAvailable;



uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

vec3 albedo = vec3(1.f);
float ambOcclusion = 1.0; 
float metallic = 0.0;
float roughness= 0.0;

vec3 getNormalFromMap(){
    vec3 tangentNormal = texture(normalTexture, vert.uv).xyz * 2.0 - 1.0;

    vec3 Q1  = dFdx(vert.position_world);
    vec3 Q2  = dFdy(vert.position_world);
    vec2 st1 = dFdx(vert.uv);
    vec2 st2 = dFdy(vert.uv);

    vec3 N   = normalize(vert.normal_world);
    vec3 T  = normalize(Q1*st2.t - Q2*st1.t);
    vec3 B  = -normalize(cross(N, T));
    mat3 TBN = mat3(T, B, N);

    return normalize(TBN * tangentNormal);
}

float distributionGGX(vec3 n, vec3 h, float roughness){
	float alpha      = roughness*roughness;
    float a2     = alpha*alpha;
    float ndh  = max(dot(n, h), 0.0);
    float ndh2 = ndh*ndh;
	
    float denom = (ndh2 * (a2 - 1.0) + 1.0);
    denom = 3.14159265359 * denom * denom;
	
    return a2 / denom;
}

float geometrySchGGX(float ndv, float roughness){
	float r = (roughness + 1.0);
    float k = (r*r) / 8.0;

    float denom = ndv * (1.0 - k) + k;
	
    return ndv / denom;
}

float geometrySm(vec3 n, vec3 v, vec3 l, float roughness){
	float ndv = max(dot(n, v), 0.0);
    float ndl = max(dot(n, l), 0.0);
    float ggx2  = geometrySchGGX(ndv, roughness);
    float ggx1  = geometrySchGGX(ndl, roughness);
	
    return ggx1 * ggx2;
}

vec3 fresnelSchlick(float cosTheta, vec3 F0){
	return F0 + (1.0 - F0) * pow(clamp(1.0 - cosTheta, 0.0, 1.0), 5.0);
}

vec3 reflectance(vec3 n, vec3 l, vec3 v, vec3 lightColor, float roughness, float metallic, vec3 F0, vec3 albedo, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	vec3 h = normalize(v+l);
	
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	
	vec3 radiance = lightColor * att;
	
	float NDF = distributionGGX(n, h, roughness);
	float G = geometrySm(n, v, l, roughness);
	vec3 F = fresnelSchlick(max(dot(h, v), 0.0), F0);

	vec3 kS = F;
	vec3 kD = vec3(1.0) - kS;
	kD *= 1.0 - metallic;

	vec3 specular = (NDF * G * F)/(4.0 * max(dot(n, v), 0.0) * max(dot(n, l), 0.0) + 0.0001);

	return (kD * albedo / 3.14159265359 + specular) * radiance * max(dot(n,l), 0.0);
}

float near = 0.06f;
float far = 100.0f;

float linearizeDepth(float depth)
{
	return (2.0 * near * far) / (far + near - (depth * 2.0 - 1.0) * (far - near));
}

float logisticDepth(float depth, float steepness, float offset)
{
	float zVal = linearizeDepth(depth);
	return (1 / (1 + exp(-steepness * (zVal - offset))));
}

void main() {	
	vec3 n = getNormalFromMap();
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 F0 = vec3(0.04); //surface reflection
    
    float depth = logisticDepth(gl_FragCoord.z, .5f, 5.f);
    vec4 background = vec4(0.45f, 0.45f, 0.45f, 1);
   
    if(textureAvailable > 1){
	
		albedo = pow((texture(albedoTexture, vert.uv).rgb), vec3(2.2));
		metallic = texture(metallicTexture, vert.uv).r;
		roughness = texture(roughTexture, vert.uv).r;
		ambOcclusion = texture(aoTexture, vert.uv).r;
        		
	}

	F0 = mix(F0, albedo, metallic);

	vec3 Lo = vec3(0.0);
	
	//directional light contribution
	Lo += reflectance(n, -dirL.direction, v, dirL.color, roughness, metallic, F0, albedo, false, vec3(0));

	//point light contribution
	Lo += reflectance(n, pointL.position - vert.position_world, v, pointL.color, roughness, metallic, F0, albedo, true, pointL.attenuation);

	vec3 fragColor = (vec3(0.03) * albedo * ambOcclusion) + Lo;
	fragColor = fragColor / (fragColor + vec3(1.0));
	fragColor = pow(fragColor, vec3(1.0f/2.0));
	
    if (fogActive)
		color = vec4(fragColor, 1.0);
    else
		color = mix(vec4(fragColor, 1.0), background, depth);
	
	brightColor = color;
    
}