#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

layout (location = 0) out vec4 color;
layout (location = 1) out vec4 brightColor;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler2D specularTexture;
uniform int textureAvailable;

uniform vec3 diffuseColor;
uniform vec3 ambientColor;
uniform vec3 specularColor;

uniform bool fogActive;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

float near = 0.06f;
float far = 100.0f;

float linearizeDepth(float depth)
{
	return (2.0 * near * far) / (far + near - (depth * 2.0 - 1.0) * (far - near));
}

float logisticDepth(float depth, float steepness, float offset)
{
	float zVal = linearizeDepth(depth);
	return (1 / (1 + exp(-steepness * (zVal - offset))));
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
    
    float depth = logisticDepth(gl_FragCoord.z, .5f, 5.f);
    vec4 background = vec4(0.45f, 0.45f, 0.45f, 1);
	
	if(textureAvailable == 1){
	
		vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	
        vec4 fragColor = vec4(texColor * materialCoefficients.x, 1); // ambient
        	
		// add directional light contribution
		fragColor.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
		// add point light contribution
		fragColor.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
        if (fogActive) {
            color = fragColor;
        }
        else {
            color = mix(fragColor, background, depth);
        }
        
        
	} else if(textureAvailable == 2) {
		
		vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	
        vec4 fragColor = vec4(texColor * materialCoefficients.x, 1); // ambient
        
		vec3 specColor = texture(specularTexture, vert.uv).rgb;
	
		// add directional light contribution
		fragColor.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color * specColor, materialCoefficients.z, specularAlpha, false, vec3(0));
			
		// add point light contribution
		fragColor.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color * specColor, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
                
        if (fogActive) {
            color = fragColor;
        }
        else {
            color = mix(fragColor, background, depth);
        }
	
	} else {
	
		vec4 fragColor = vec4(diffuseColor * materialCoefficients.x, 1); // ambient
        
		// add directional light contribution
		fragColor.rgb += phong(n, -dirL.direction, v, dirL.color * diffuseColor, materialCoefficients.y, dirL.color * specularColor, materialCoefficients.z, specularAlpha, false, vec3(0));
			
		// add point light contribution
		fragColor.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * diffuseColor, materialCoefficients.y, pointL.color * specularColor, materialCoefficients.z, specularAlpha, true, pointL.attenuation);

        if (fogActive) {
            color = fragColor;
        }
        else {
            color = mix(fragColor, background, depth);
        }
	}
	
	brightColor = color;
}

